@@ -103,7 +103,6 @@ class ConjectureRunner(object):
 
         self.health_check_state = None
 
-        self.used_examples_from_database = False
         self.tree = DataTree()
 
         self.best_observed_targets = defaultdict(lambda: NO_SCORE)
@@ -138,14 +137,21 @@ class ConjectureRunner(object):
         assert isinstance(data.observer, TreeRecordingObserver)
         self.call_count += 1
 
+        interrupted = False
         try:
             self.__stoppable_test_function(data)
+        except KeyboardInterrupt:
+            interrupted = True
+            raise
         except BaseException:
             self.save_buffer(data.buffer)
             raise
         finally:
-            data.freeze()
-            self.note_details(data)
+            # No branch, because if we're interrupted we always raise
+            # the KeyboardInterrupt, never continue to the code below.
+            if not interrupted:  # pragma: no branch
+                data.freeze()
+                self.note_details(data)
 
         self.debug_data(data)
 
@@ -430,8 +436,6 @@ class ConjectureRunner(object):
                     extra.sort(key=sort_key)
                     corpus.extend(extra)
 
-            self.used_examples_from_database = len(corpus) > 0
-
             for existing in corpus:
                 last_data = ConjectureData.for_buffer(
                     existing, observer=self.tree.new_observer()
@@ -548,7 +552,7 @@ class ConjectureRunner(object):
         """If the data or result object is suitable for hill climbing, run hill
         climbing on all of its target observations."""
         if data.status == Status.VALID:
-            for target, score in data.target_observations.items():
+            for target in data.target_observations:
                 self.new_optimiser(data, target).run()
 
     def _run(self):
@@ -694,7 +698,7 @@ class ConjectureRunner(object):
             data = self.new_conjecture_data_for_buffer(buffer)
             self.test_function(data)
             result = check_result(data.as_result())
-            assert status is None or result.status == status
+            assert status is None or result.status == status, (status, result.status)
             status = result.status
         if status == Status.OVERRUN:
             result = Overrun
