@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -20,15 +20,15 @@ from __future__ import absolute_import, division, print_function
 import enum
 import hashlib
 import heapq
+import sys
 from collections import OrderedDict
 from fractions import Fraction
 
-from hypothesis._settings import note_deprecation
+from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import (
     abc,
     bit_length,
     floor,
-    hbytes,
     hrange,
     int_from_bytes,
     qualname,
@@ -40,7 +40,7 @@ LABEL_MASK = 2 ** 64 - 1
 
 
 def calc_label_from_name(name):
-    hashed = hashlib.md5(str_to_bytes(name)).digest()
+    hashed = hashlib.sha384(str_to_bytes(name)).digest()
     return int_from_bytes(hashed[:8])
 
 
@@ -69,7 +69,7 @@ def integer_range(data, lower, upper, center=None):
         # on other values we don't suddenly disappear when the gap shrinks to
         # zero - if that happens then often the data stream becomes misaligned
         # and we fail to shrink in cases where we really should be able to.
-        data.write(hbytes([0]))
+        data.draw_bits(1, forced=0)
         return int(lower)
 
     if center is None:
@@ -93,6 +93,14 @@ def integer_range(data, lower, upper, center=None):
     bits = bit_length(gap)
     probe = gap + 1
 
+    if bits > 24 and data.draw_bits(3):
+        # For large ranges, we combine the uniform random distribution from draw_bits
+        # with the weighting scheme used by WideRangeIntStrategy with moderate chance.
+        # Cutoff at 2 ** 24 so unicode choice is uniform but 32bit distribution is not.
+        idx = Sampler([4.0, 8.0, 1.0, 1.0, 0.5]).sample(data)
+        sizes = [8, 16, 32, 64, 128]
+        bits = min(bits, sizes[idx])
+
     while probe > gap:
         data.start_example(INTEGER_RANGE_DRAW_LABEL)
         probe = data.draw_bits(bits)
@@ -107,29 +115,21 @@ def integer_range(data, lower, upper, center=None):
     return int(result)
 
 
-try:
-    from numpy import ndarray
-except ImportError:  # pragma: no cover
-    ndarray = ()
-
-
 def check_sample(values, strategy_name):
-    if isinstance(values, ndarray):
+    if "numpy" in sys.modules and isinstance(values, sys.modules["numpy"].ndarray):
         if values.ndim != 1:
-            note_deprecation(
+            raise InvalidArgument(
                 (
                     "Only one-dimensional arrays are supported for sampling, "
                     "and the given value has {ndim} dimensions (shape "
                     "{shape}).  This array would give samples of array slices "
                     "instead of elements!  Use np.ravel(values) to convert "
                     "to a one-dimensional array, or tuple(values) if you "
-                    "want to sample slices.  Sampling a multi-dimensional "
-                    "array will be an error in a future version of Hypothesis."
-                ).format(ndim=values.ndim, shape=values.shape),
-                since="2018-05-09",
+                    "want to sample slices."
+                ).format(ndim=values.ndim, shape=values.shape)
             )
     elif not isinstance(values, (OrderedDict, abc.Sequence, enum.EnumMeta)):
-        note_deprecation(
+        raise InvalidArgument(
             "Cannot sample from {values}, not an ordered collection. "
             "Hypothesis goes to some length to ensure that the {strategy} "
             "strategy has stable results between runs. To replay a saved "
@@ -140,8 +140,7 @@ def check_sample(values, strategy_name):
             "handling - and note that when simplifying an example, "
             "Hypothesis treats earlier values as simpler.".format(
                 values=repr(values), strategy=strategy_name
-            ),
-            since="2017-04-12",
+            )
         )
     return tuple(values)
 
@@ -150,19 +149,12 @@ def choice(data, values):
     return values[integer_range(data, 0, len(values) - 1)]
 
 
-def getrandbits(data, n):
-    n_bytes = n // 8
-    if n % 8 != 0:
-        n_bytes += 1
-    return int_from_bytes(data.draw_bytes(n_bytes)) & ((1 << n) - 1)
-
-
 FLOAT_PREFIX = 0b1111111111 << 52
 FULL_FLOAT = int_to_float(FLOAT_PREFIX | ((2 << 53) - 1)) - 1
 
 
 def fractional_float(data):
-    return (int_to_float(FLOAT_PREFIX | getrandbits(data, 52)) - 1) / FULL_FLOAT
+    return (int_to_float(FLOAT_PREFIX | data.draw_bits(52)) - 1) / FULL_FLOAT
 
 
 def boolean(data):
@@ -170,7 +162,7 @@ def boolean(data):
 
 
 def biased_coin(data, p):
-    """Return False with probability p (assuming a uniform generator),
+    """Return True with probability p (assuming a uniform generator),
     shrinking towards False."""
     data.start_example(BIASED_COIN_LABEL)
     while True:
@@ -193,10 +185,10 @@ def biased_coin(data, p):
         # to write a byte to the data stream anyway so that these don't cause
         # difficulties when shrinking.
         if p <= 0:
-            data.write(hbytes([0]))
+            data.draw_bits(1, forced=0)
             result = False
         elif p >= 1:
-            data.write(hbytes([1]))
+            data.draw_bits(1, forced=1)
             result = True
         else:
             falsey = floor(256 * (1 - p))
